#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
import urllib2
import sys
import time

BUFFER_SIZE = 500
LEAVE_OUT = 1 # that does not leave out anything

address = 'http://127.0.0.1:10001'


def file_len(fname):
    with open(fname) as f:
        for i, l in enumerate(f):
            pass
    return i + 1

def add_examples(examples, tmId):
    examplesData = {
    'operation': 'addSentences',
    'tmId':tmId,
    'examples':examples
    }
    req = urllib2.Request(address)
    req.add_header('Content-Type', 'application/json')
    response = json.loads(urllib2.urlopen(req, json.dumps(examplesData), timeout = 3600).read())
    #print(response)
    if response['status'] == 'error':
        raise Exception(response['message'])

if len(sys.argv) != 10:
    raise Exception("wrong number of arguments")

name = sys.argv[1]
sourceFile = sys.argv[2]
tokenizedSourceFile = sys.argv[3]
lemmatizedSourceFile = sys.argv[4]
sourceLangId = int(sys.argv[5])
targetFile = sys.argv[6]
targetLangId = int(sys.argv[7])
alignmentsFile = sys.argv[8]
sourceIdsFile = sys.argv[9]

sourceFileLength = file_len(sourceFile)
tokenizedSourceFileLength = file_len(tokenizedSourceFile)
lemmatizedSourceFileLength = file_len(lemmatizedSourceFile)
targetFileLength = file_len(targetFile)
alignmentsFileLength = file_len(alignmentsFile)
sourceIdsFileLength = file_len(sourceIdsFile)

if not (sourceFileLength == tokenizedSourceFileLength and tokenizedSourceFileLength == lemmatizedSourceFileLength and lemmatizedSourceFileLength == targetFileLength and targetFileLength == alignmentsFileLength and alignmentsFileLength == sourceIdsFileLength):
    print("File lengths:")
    print("source file: %d\nlemmatized source file: %d\ntarget file: %d\nalignments file: %d\nsource ids file: %d" % (sourceFileLength, lemmatizedSourceFileLength, targetFileLength, alignmentsFileLength, sourceIdsFileLength))
    raise Exception("files are not of the same length!")

totalExamples = sourceFileLength / LEAVE_OUT

data = {
    'operation': 'addPairedTms',
    'sourceLangId':sourceLangId,
    'targetLangId':targetLangId,
    'name':name
}

req = urllib2.Request(address)
req.add_header('Content-Type', 'application/json')
response = json.loads(urllib2.urlopen(req, json.dumps(data), timeout = 3600).read())
#print(response)

lemmatizedTmId = int(response['lemmatizedTmId'])
nonLemmatizedTmId = int(response['nonLemmatizedTmId'])
print "Added new paired tm: lemmatized id: %d, non lemmatized id: %d" % (lemmatizedTmId, nonLemmatizedTmId)

examples = []
examples_lemmatized = []
start = time.time()
with open(sourceFile) as source_file, open(tokenizedSourceFile) as tokenized_source_file, open(lemmatizedSourceFile) as lemmatized_source_file, open(targetFile) as target_file, open(alignmentsFile) as alignments_file, open(sourceIdsFile) as source_ids_file:
    addedCount = 0
    for lineNumber in range(sourceFileLength):
        if lineNumber % LEAVE_OUT == 0:
            sourceSentence = source_file.readline().strip()
            tokenizedSourceSentence = tokenized_source_file.readline().strip()
            lemmatizedSourceSentence = lemmatized_source_file.readline().strip()
            targetSentence = target_file.readline().strip()
            alignment = json.loads(alignments_file.readline().strip())
            sourceId = int(source_ids_file.readline().strip())

            examples.append([sourceSentence, tokenizedSourceSentence, targetSentence, alignment, sourceId])
            examples_lemmatized.append([sourceSentence, lemmatizedSourceSentence, targetSentence, alignment, sourceId])
            addedCount += 1
            if len(examples) >= BUFFER_SIZE:
                add_examples(examples, nonLemmatizedTmId)
                add_examples(examples_lemmatized, lemmatizedTmId)
                mark = time.time()
                print "Added %d of %d examples. Time elapsed: %.4f s, current speed: %.4f examples/second" % (addedCount, totalExamples, mark-start, addedCount/(mark-start))
                examples = []
                examples_lemmatized = []


if len(examples) > 0:
    add_examples(examples, nonLemmatizedTmId)
    add_examples(examples_lemmatized, lemmatizedTmId)

end = time.time()
print "Added all %d sentences. Time elapsed: %.4f s, overall speed: %.4f sentences/second" % (addedCount, end-start, addedCount/(end-start))

print "Generating indexes..."
start = time.time()
data = {
    'operation': 'refreshIndex',
    'tmId' : nonLemmatizedTmId
}
req = urllib2.Request(address)
req.add_header('Content-Type', 'application/json')
urllib2.urlopen(req, json.dumps(data), timeout = 3600).read()

data = {
    'operation': 'refreshIndex',
    'tmId' : lemmatizedTmId
}
req = urllib2.Request(address)
req.add_header('Content-Type', 'application/json')
urllib2.urlopen(req, json.dumps(data), timeout = 3600).read()

end = time.time()
print "Index regeneration complete. The operation took %.4f s" % (end - start)